@@ -19,7 +19,7 @@ static int cleanup_dir(void)
         logfile(LOG_WARNING, _("Unable to opendir : [%s]"),
                 strerror(errno));
         return -1;
-    }    
+    }
     while ((entry = readdir(dir)) != NULL) {
         if (stat(entry->d_name, &st) != 0 || !S_ISREG(st.st_mode)) {
             continue;
@@ -30,18 +30,24 @@ static int cleanup_dir(void)
         }
     }
     (void) closedir(dir);
-    
+
     return 0;
 }
+
+static void check_parent(void)
+{
+    pid_t ppid;
     
+    if ((ppid = getppid()) < (pid_t) 2 || kill(ppid, 0) != 0) {
+        exit(EXIT_SUCCESS);
+    }
+}
+
 int expire(void)
 {
     nice(EXPIRE_NICENESS);
-    if (chdir(storage_dir) != 0) {
-        logfile(LOG_ERR, _("Unable to chdir to [%s]"), storage_dir);
-        return -1;        
-    }
     for (;;) {
+        check_parent();
         logfile(LOG_DEBUG, _("Pruning [%s] directory"), storage_dir);
         cleanup_dir();
         sleep(EXPIRE_PAUSE);
