@@ -17,7 +17,7 @@ void fd_nonblock(const int fd)
 
 int safe_write(const int fd, const void *buf_, size_t count)
 {
-    ssize_t written;    
+    ssize_t written;
     register const char *buf = buf_;
 
     while (count > (size_t) 0U) {
@@ -92,11 +92,11 @@ static void returncode_bufferev_error_cb(struct bufferevent * const bufferev,
                                          short what, void *client_)
 {
     Client * const client = client_;
-    
+
     (void) bufferev;
     logfile(LOG_DEBUG, _("Error %d when sending return code to fd #%d"),
             (int) what, client->client_fd);
-    client_disconnect(client);    
+    client_disconnect(client);
 }
 
 static int validate_key(const unsigned char *buf, size_t size)
@@ -111,21 +111,25 @@ static int validate_key(const unsigned char *buf, size_t size)
             return -1;
         }
     } while (size > (size_t) 0U);
-    
+
     return 0;
 }
 
 static int client_process_store(Client * const client)
 {
     char *store_file;
+    char *store_file_tmp;
     char *store_file_pnt;
+    char *store_file_tmp_pnt;
     size_t sizeof_store_file;
+    size_t sizeof_store_file_tmp;
 
     logfile(LOG_DEBUG, _("client_process_store, fd #%d"), client->client_fd);
     if (client->offset_read_buf < (size_t) 10U) {
         return -1;
     }
-    sizeof_store_file = (sizeof "/") - (size_t) 1U + client->key_len + (size_t) 1U;
+    sizeof_store_file = (sizeof "/") - (size_t) 1U +
+        client->key_len + (size_t) 1U;
     if ((store_file = ALLOCA(sizeof_store_file)) == NULL) {
         logfile(LOG_ERR, _("Out of stack for ALLOCA"));
         return -1;
@@ -133,31 +137,64 @@ static int client_process_store(Client * const client)
     store_file_pnt = store_file;
     *store_file_pnt++ = '/';
     if (validate_key(client->read_buf + (size_t) 9U, client->key_len) != 0) {
-        ALLOCA_FREE(store_file);        
+        ALLOCA_FREE(store_file);
         logfile(LOG_WARNING, _("Invalid key name"));
         return -1;
     }
     memcpy(store_file_pnt, client->read_buf + (size_t) 9U, client->key_len);
     store_file_pnt += client->key_len;
     *store_file_pnt = 0;
-    logfile(LOG_DEBUG, _("Creating [%s]"), store_file);
+
+    sizeof_store_file_tmp = (sizeof "/") - (size_t) 1U +
+        sizeof STOREFILE_TMP_PREFIX - (size_t) 1U +
+        client->key_len + (size_t) 1U;
+    if ((store_file_tmp = ALLOCA(sizeof_store_file_tmp)) == NULL) {
+        logfile(LOG_ERR, _("Out of stack for ALLOCA"));
+        ALLOCA_FREE(store_file);
+        return -1;
+    }
+    store_file_tmp_pnt = store_file_tmp;
+    *store_file_tmp_pnt++ = '/';
+    memcpy(store_file_tmp_pnt, STOREFILE_TMP_PREFIX,
+           sizeof STOREFILE_TMP_PREFIX - (size_t) 1U);
+    store_file_tmp_pnt += sizeof STOREFILE_TMP_PREFIX - (size_t) 1U;
+    memcpy(store_file_tmp_pnt,
+           client->read_buf + (size_t) 9U, client->key_len);
+    store_file_tmp_pnt += client->key_len;
+    *store_file_tmp_pnt = 0;
+
+    logfile(LOG_DEBUG, _("Creating [%s]"), store_file_tmp);
     if ((client->write_fd =
-         open(store_file, O_CREAT | O_NOFOLLOW | O_WRONLY,
+         open(store_file_tmp, O_CREAT | O_NOFOLLOW | O_WRONLY | O_TRUNC,
               (mode_t) 0600)) == -1) {
         logfile(LOG_WARNING, _("Unable to create [%s]: [%s]"), store_file,
                 strerror(errno));
         ALLOCA_FREE(store_file);
+        ALLOCA_FREE(store_file_tmp);
         return -1;
     }
     logfile(LOG_DEBUG, _("[%s] = fd #%d"), store_file, client->write_fd);
-    ALLOCA_FREE(store_file);
     if (safe_write(client->write_fd,
                    client->read_buf + (size_t) 9U + client->key_len,
                    client->data_len) < 0) {
         logfile(LOG_WARNING, _("Write error: [%s]"), strerror(errno));
     }
-    if ((client->returncode_bufev = 
-         bufferevent_new(client->client_fd, 
+    (void) close(client->write_fd);
+    client->write_fd = -1;
+    logfile(LOG_DEBUG,
+            _("Renaming [%s] to [%s]"), store_file_tmp, store_file);
+    if (rename(store_file_tmp, store_file) != 0) {
+        logfile(LOG_WARNING, _("Unable to rename [%s] to [%s]: [%s]"),
+                store_file_tmp, store_file, strerror(errno));
+        (void) unlink(store_file_tmp);
+        ALLOCA_FREE(store_file);
+        ALLOCA_FREE(store_file_tmp);
+        return -1;
+    }
+    ALLOCA_FREE(store_file);
+    ALLOCA_FREE(store_file_tmp);
+    if ((client->returncode_bufev =
+         bufferevent_new(client->client_fd,
                          returncode_bufferev_read_cb,
                          returncode_bufferev_write_cb,
                          returncode_bufferev_error_cb,
@@ -181,8 +218,9 @@ static int client_process_fetch(Client * const client)
     logfile(LOG_DEBUG, _("client_process_fetch, fd #%d"), client->client_fd);
     if (client->offset_read_buf < (size_t) 6U) {
         return -1;
-    }    
-    sizeof_store_file = (sizeof "/") - (size_t) 1U + client->key_len + (size_t) 1U;
+    }
+    sizeof_store_file = (sizeof "/") - (size_t) 1U +
+        client->key_len + (size_t) 1U;
     if ((store_file = ALLOCA(sizeof_store_file)) == NULL) {
         logfile(LOG_WARNING, _("Out of stack for ALLOCA"));
         return -1;
@@ -190,7 +228,7 @@ static int client_process_fetch(Client * const client)
     store_file_pnt = store_file;
     *store_file_pnt++ = '/';
     if (validate_key(client->read_buf + (size_t) 5U, client->key_len) != 0) {
-        ALLOCA_FREE(store_file);        
+        ALLOCA_FREE(store_file);
         logfile(LOG_WARNING, _("Invalid key name"));
         return -1;
     }
@@ -199,20 +237,20 @@ static int client_process_fetch(Client * const client)
     *store_file_pnt = 0;
     logfile(LOG_DEBUG, _("Reading [%s]"), store_file);
     if ((client->read_fd = open(store_file, O_RDONLY)) == -1) {
-        logfile(LOG_WARNING, _("Unable to fetch [%s]: [%s]"), store_file,
+        logfile(LOG_DEBUG, _("Unable to fetch [%s]: [%s]"), store_file,
                 strerror(errno));
         ALLOCA_FREE(store_file);
         return -1;
     }
     logfile(LOG_DEBUG, _("[%s] = fd #%d"), store_file, client->read_fd);
     ALLOCA_FREE(store_file);
-    if (fstat(client->read_fd, &st) != 0) {
+    if (fstat(client->read_fd, &st) != 0 || st.st_size <= (off_t) 0) {        
         logfile(LOG_DEBUG, _("Unable to stat a previous key (%s) : [%s]"),
                 store_file, strerror(errno));
         (void) close(client->read_fd);
         client->read_fd = -1;
         ALLOCA_FREE(store_file);
-        return -1;        
+        return -1;
     }
     if ((client->read_mapped_zone = mmap(NULL, st.st_size, PROT_READ,
                                          MAP_SHARED, client->read_fd,
@@ -225,8 +263,8 @@ static int client_process_fetch(Client * const client)
         return -1;
     }
     client->read_mapped_zone_length = st.st_size;
-    if ((client->returncode_bufev = 
-         bufferevent_new(client->client_fd, 
+    if ((client->returncode_bufev =
+         bufferevent_new(client->client_fd,
                          returncode_bufferev_read_cb,
                          returncode_bufferev_write_cb,
                          returncode_bufferev_error_cb,
@@ -237,7 +275,7 @@ static int client_process_fetch(Client * const client)
     (void) bufferevent_write(client->returncode_bufev,
                              client->read_mapped_zone,
                              client->read_mapped_zone_length);
-    
+
     return 0;
 }
 
@@ -250,8 +288,9 @@ static int client_process_delete(Client * const client)
     logfile(LOG_DEBUG, _("client_process_delete, fd #%d"), client->client_fd);
     if (client->offset_read_buf < (size_t) 6U) {
         return -1;
-    }    
-    sizeof_store_file = (sizeof "/") - (size_t) 1U + client->key_len + (size_t) 1U;
+    }
+    sizeof_store_file = (sizeof "/") - (size_t) 1U +
+        client->key_len + (size_t) 1U;
     if ((store_file = ALLOCA(sizeof_store_file)) == NULL) {
         logfile(LOG_WARNING, _("Out of stack for ALLOCA"));
         return -1;
@@ -259,7 +298,7 @@ static int client_process_delete(Client * const client)
     store_file_pnt = store_file;
     *store_file_pnt++ = '/';
     if (validate_key(client->read_buf + (size_t) 5U, client->key_len) != 0) {
-        ALLOCA_FREE(store_file);        
+        ALLOCA_FREE(store_file);
         logfile(LOG_WARNING, _("Invalid key name"));
         return -1;
     }
@@ -273,8 +312,8 @@ static int client_process_delete(Client * const client)
         ALLOCA_FREE(store_file);
         return -1;
     }
-    if ((client->returncode_bufev = 
-         bufferevent_new(client->client_fd, 
+    if ((client->returncode_bufev =
+         bufferevent_new(client->client_fd,
                          returncode_bufferev_read_cb,
                          returncode_bufferev_write_cb,
                          returncode_bufferev_error_cb,
@@ -284,7 +323,7 @@ static int client_process_delete(Client * const client)
     }
     (void) bufferevent_write(client->returncode_bufev, (void *) RETURNCODE_OK,
                              sizeof RETURNCODE_OK - (size_t) 1U);
-    
+
     return 0;
 }
 
@@ -297,12 +336,12 @@ static void client_read(const int client_fd, short event, void *client_)
     if (event == EV_TIMEOUT) {
         logfile(LOG_DEBUG, _("Timeout on descriptor #%d"), client->client_fd);
         client_disconnect(client);
-        return;                
+        return;
     }
     needed_size = client->offset_read_buf + read_chunk_size;
     if (client->sizeof_read_buf < needed_size) {
         unsigned char *tmp_buf;
-        
+
         if ((tmp_buf = realloc(client->read_buf,
                                needed_size)) == NULL) {
             logfile(LOG_ERR, _("Out of memory for client read buffers"));
@@ -317,7 +356,7 @@ static void client_read(const int client_fd, short event, void *client_)
          client->read_buf + client->offset_read_buf,
          client->sizeof_read_buf - client->offset_read_buf);
     if (readen <= (ssize_t) 0) {
-        if (errno == EINTR || errno == EAGAIN) {            
+        if (errno == EINTR || errno == EAGAIN) {
             logfile(LOG_DEBUG, _("Interrupted read for fd #%d: [%s]"),
                     client_fd, strerror(errno));
             event_add(&client->ev_client_read, &timeout);
@@ -332,12 +371,12 @@ static void client_read(const int client_fd, short event, void *client_)
                     strerror(errno));
         }
         client_disconnect(client);
-        return;        
+        return;
     }
     client->offset_read_buf += readen;
     logfile(LOG_DEBUG, _("Activity on client #%d (%d) offset=%lu"),
             client->client_fd, event,
-            (unsigned int) client->offset_read_buf);    
+            (unsigned int) client->offset_read_buf);
     if (client->client_command == CC_UNDEF) {
         if (client->read_buf[0] == CC_FETCH &&
             client->offset_read_buf > 5) {
@@ -370,11 +409,11 @@ static void client_read(const int client_fd, short event, void *client_)
             logfile(LOG_WARNING, _("Unknown command [%d], good bye."),
                     (int) client->read_buf[0]);
             client_disconnect(client);
-            return;            
+            return;
         }
     }
     if (client->total_len > (size_t) 0U &&
-        client->offset_read_buf == client->total_len) {        
+        client->offset_read_buf == client->total_len) {
         if (client->client_command == CC_STORE) {
             if (client_process_store(client) < 0) {
                 client_disconnect(client);
@@ -417,7 +456,7 @@ static void client_read(const int client_fd, short event, void *client_)
 }
 
 void new_client(const int listen_fd, short event, void *ev)
-{    
+{
     Client *client;
     struct sockaddr_storage client_sa;
     socklen_t client_sa_len = sizeof client_sa;
@@ -451,7 +490,7 @@ void new_client(const int listen_fd, short event, void *ev)
     client->offset_read_buf = (size_t) 0U;
     client->client_command = CC_UNDEF;
     client->key_len = (size_t) 0U;
-    client->data_len = (size_t) 0U;    
+    client->data_len = (size_t) 0U;
     client->total_len = (size_t) 0U;
     client->write_fd = -1;
     client->read_fd = -1;
@@ -461,5 +500,5 @@ void new_client(const int listen_fd, short event, void *ev)
     event_set(&client->ev_client_read, client_fd, EV_READ,
               client_read, client);
     event_add(&client->ev_client_read, &timeout);
-    logfile(LOG_DEBUG, _("New client on fd #%d"), client_fd);    
+    logfile(LOG_DEBUG, _("New client on fd #%d"), client_fd);
 }
